
import com.apple.yellow.foundation.*;
import com.apple.yellow.webobjects.*;
import com.apple.yellow.eocontrol.*;
import com.apple.yellow.eoaccess.*;
import com.apple.webobjects.xml.*;



public class Syndication extends WODirectAction {

    // Add an ivar to store the coder we will use (and lazy initialize) ... protected WOXMLCoder coder;
    protected WOXMLCoder coder;

    public Syndication(WORequest aRequest) {
        super(aRequest);
    }

    public WOComponent defaultAction() {
        return pageWithName("Main");
    }

    public WOMessage allMessagesAction() {
        // Get the client's name
        String clientName = (String)request().formValueForKey("client");
        // Fetch the client object
        EOEnterpriseObject client = EOUtilities.objectMatchingKeyAndValue(session().defaultEditingContext(), "Client","name", clientName);
        // Then we fetch the articles based on the date
        NSArray articles = EOUtilities.objectsForEntityNamed(session().defaultEditingContext(), "Article");


        // Now we encode the information in XML
        WOResponse response = encodeArticleArray(articles);


        // Before we return, we should update the "lastRead" time stamp.
        timeStampClient(client, articles);
        return response;
    }
    
    public WOMessage newMessagesAction() {
        // Get the client's name
        String clientName = (String)request().formValueForKey("client");
        // Fetch the client object
        EOEnterpriseObject client = EOUtilities.objectMatchingKeyAndValue(session().defaultEditingContext(), "Client", "name", clientName);
        NSDictionary queryDictionary = null;
        if( client.valueForKey("lastRead") != null ) {
            queryDictionary = new NSDictionary(client.valueForKey("lastRead"), "date");
        };
        // Then we fetch the articles based on the date
        NSArray articles = EOUtilities.objectsWithFetchSpecificationAndBindings(session().defaultEditingContext(),"Article","NewArticles", queryDictionary);

        
        // We encode the articles as XML
        WOResponse response = encodeArticleArray(articles);

        
        // Before we return, we should update the "lastRead" time stamp.
        timeStampClient(client, articles);
        // Let's logthe XML for demo purposes
        logString("Vending XML: \n" + (new String(response.content().bytes(0, response.content().length()))));
        return response;
    }

    // This method just saves the current time into the database.
    protected void timeStampClient(EOEnterpriseObject client, NSArray articles) {
        NSGregorianDate date = null;
        java.util.Enumeration e = articles.objectEnumerator();
        while( e.hasMoreElements() ) {
            EOEnterpriseObject article = (EOEnterpriseObject)e.nextElement();
            if( date == null ) {
                date = (NSGregorianDate)article.valueForKey("publishDate");
            } else {
                date = (NSGregorianDate) date.laterDate((NSGregorianDate)article.valueForKey("publishDate"));
            }
        }
        if( date != null ) {
            client.takeValueForKey( date, "lastRead");
            logString("Our client looks like " + client);
            try {
                session().defaultEditingContext().saveChanges();
            } catch (Exception exception) {
                logString("Syndication::timeStampClient - exception saving 'client' \n" + exception);
            }
        }
    }


/*
    protected WOResponse encodeArticleArray(NSArray articles) {
        // Check if we have a coder, create one if we don't
        
        // Encode the objects into XML
        
        // Create a response

        // Append the "XML Header"

        // Append our encoded articles to the reponse

        // Never forget to send out the MIME type

        
        return response;
    }
*/

    
    


    protected WOResponse encodeArticleArray(NSArray articles) {
        // Check if we have a coder, create one if we don't
        if (coder == null ) {
            // coder = WOXMLCoder.coder();
            // There is a bug?? in the default WOXMLCoder implementation that
            // encodes the date using an attached date formatter.  DTW attaches the
            // date formatter used when the date is created (the format string you see in the UI)
            // So, we have a custom subclass of WOXMLCoder that forces the "generic" date formatter to be used
            coder = new FixedDateXMLCoder();
        }

        // Encode the objects into XML
        String xmlString = coder.encodeRootObjectForKey(articles, "articles");

        // Create a response
        WOResponse response = new WOResponse();
        // Append the "XML Header"
        response.appendContentString("<?xml version=\"1.0\" ?>");
        // Append our encoded articles to the reponse
        response.appendContentString(xmlString);
        // Never forget to send out the MIME type
        response.setHeader("text/xml", "content-type");

        return response;
    }

}


//  "<?xml version=\"1.0\" ?>"
//  response.appendContentString("<?xml version=\"1.0\" ?>");

